@@ -8,7 +8,8 @@ from z3c.saconfig.interfaces import ISiteScopedSession
 from zope import component
 from zope.event import notify
 from zope.interface import implementer
-from zope.sqlalchemy import ZopeTransactionExtension
+from zope.sqlalchemy import register
+
 
 import sqlalchemy
 import threading
@@ -24,7 +25,7 @@ except ImportError:
 SESSION_DEFAULTS = dict(
     autocommit=False,
     autoflush=True,
-    extension=ZopeTransactionExtension())
+    )
 
 
 @implementer(IScopedSession)
@@ -66,7 +67,9 @@ class GloballyScopedSession(object):
             engine_factory = component.getUtility(IEngineFactory,
                                                   name=self.engine)
             kw['bind'] = engine_factory()
-        return sqlalchemy.orm.create_session(**kw)
+        session = sqlalchemy.orm.create_session(**kw)
+        register(session)
+        return session
 
     def scopeFunc(self):
         return get_ident()
@@ -104,7 +107,9 @@ class SiteScopedSession(object):
                                               name=self.engine)
         kw = self.kw.copy()
         kw['bind'] = engine_factory()
-        return sqlalchemy.orm.create_session(**kw)
+        session = sqlalchemy.orm.create_session(**kw)
+        register(session)
+        return session
 
     def scopeFunc(self):
         return (get_ident(), self.siteScopeFunc())
