@@ -1,21 +1,25 @@
+from .interfaces import IEngineFactory
+from .interfaces import IScopedSession
+from .utility import EngineFactory
+from zope.configuration.name import resolve
+
+import zope.component.zcml
 import zope.interface
 import zope.schema
-import zope.component.zcml
+
+
 try:
     from zope.component.security import PublicPermission
 except ImportError:
     # BBB for Zope 2.10
     from zope.component.zcml import PublicPermission
-from zope.configuration.name import resolve
 
-import utility
-import interfaces
 
 class IEngineDirective(zope.interface.Interface):
     """Registers a database engine factory."""
 
     url = zope.schema.URI(
-        title=u"Database URL",
+        title=u'Database URL',
         description=u"e.g. 'sqlite:///:memory:'.",
         required=True)
 
@@ -23,7 +27,7 @@ class IEngineDirective(zope.interface.Interface):
         title=u"Engine name",
         description=u"Empty if this is the default engine.",
         required=False,
-        default=u"")
+        default=u'')
 
     convert_unicode = zope.schema.Bool(
         title=u'Convert all string columns to unicode',
@@ -33,44 +37,49 @@ class IEngineDirective(zope.interface.Interface):
                     u'UnicodeString columns',
         required=False,
         default=False)
-    
+
     echo = zope.schema.Bool(
         title=u'Echo SQL statements',
         description=u'Enable logging statements for debugging.',
         required=False,
         default=None)
-    
+
     setup = zope.schema.BytesLine(
         title=u'After engine creation hook',
-        description=u'Callback for creating mappers etc. One argument is passed, the engine',
+        description=u'Callback for creating mappers etc. '
+                    u'One argument is passed, the engine',
         required=False,
         default=None)
 
     # Connection pooling options - probably only works on SQLAlchemy 0.5 and up
-    
+
     pool_size = zope.schema.Int(
         title=u"The size of the pool to be maintained",
         description=u"Defaults to 5 in SQLAlchemy.",
         required=False)
-    
+
     max_overflow = zope.schema.Int(
         title=u"The maximum overflow size of the pool.",
-        description=u"When the number of checked-out connections reaches the " +
-                    u"size set in pool_size, additional connections will be " +
-                    u"returned up to this limit. Defaults to 10 in SQLAlchemy",
+        description=u"When the number of checked-out connections "
+                    u"reaches the size set in pool_size, additional "
+                    u"connections will be returned up to this limit. "
+                    u"Defaults to 10 in SQLAlchemy",
         required=False)
-    
+
     pool_recycle = zope.schema.Int(
         title=u"Number of seconds between connection recycling",
-        description=u"Upon checkout, if this timeout is surpassed the connection "
-                    u"will be closed and replaced with a newly opened connection",
+        description=u"Upon checkout, if this timeout is "
+                    u"surpassed the connection will be closed and "
+                    u"replaced with a newly opened connection",
         required=False)
-    
+
     pool_timeout = zope.schema.Int(
-        title=u"The number of seconds to wait before giving up on returning a connection.",
+        title=u"The number of seconds to wait before giving up on "
+              u"returning a connection.",
         description=u"Defaults to 30 in SQLAlchemy if not set",
         required=False)
 
+
 class ISessionDirective(zope.interface.Interface):
     """Registers a database scoped session"""
 
@@ -98,14 +107,17 @@ class ISessionDirective(zope.interface.Interface):
         required=False,
         default="z3c.saconfig.utility.GloballyScopedSession")
 
-def engine(_context, url, name=u"", convert_unicode=False, echo=None, setup=None, twophase=False,
-    pool_size=None, max_overflow=None, pool_recycle=None, pool_timeout=None):
-    
+
+def engine(_context, url, name=u"", convert_unicode=False,
+           echo=None, setup=None, twophase=False,
+           pool_size=None, max_overflow=None, pool_recycle=None,
+           pool_timeout=None):
+
     kwargs = {
         'echo': echo,
         'convert_unicode': convert_unicode,
     }
-    
+
     # Only add these if they're actually set, since we want to let SQLAlchemy
     # control the defaults
     if pool_size is not None:
@@ -116,22 +128,30 @@ def engine(_context, url, name=u"", convert_unicode=False, echo=None, setup=None
         kwargs['pool_recycle'] = pool_recycle
     if pool_timeout is not None:
         kwargs['pool_timeout'] = pool_timeout
-    
-    factory = utility.EngineFactory(url, **kwargs)
-    
+
+    factory = EngineFactory(url, **kwargs)
+
     zope.component.zcml.utility(
         _context,
-        provides=interfaces.IEngineFactory,
+        provides=IEngineFactory,
         component=factory,
         permission=PublicPermission,
         name=name)
-    
+
     if setup:
+        if isinstance(setup, bytes):
+            setup = setup.decode()
+
         if _context.package is None:
             callback = resolve(setup)
         else:
             callback = resolve(setup, package=_context.package.__name__)
-        callback(factory())
+        _context.action(
+            discriminator=(IEngineFactory, name),
+            callable=callback,
+            args=(factory(), ),
+            order=9999)
+
 
 def session(_context, name=u"", engine=u"", twophase=False,
             factory="z3c.saconfig.utility.GloballyScopedSession"):
@@ -143,7 +163,7 @@ def session(_context, name=u"", engine=u"", twophase=False,
 
     zope.component.zcml.utility(
         _context,
-        provides=interfaces.IScopedSession,
+        provides=IScopedSession,
         component=scoped_session,
         permission=PublicPermission,
         name=name)
